import json

import pandas as pd
from constants import *


class Summation:
    def __init__(self, start_month: str, start_year: int, end_month: str, end_year: int, save_file: bool):
        """This class will summate the data given in a variety of different
        formats.
        :param start_month: The month where the time span begins (in 3 letter abbreviation) :type start_month: str
        :param start_year: The year where the time span begins (YYYY) :type start_year: int
        :param end_month: The month where the time span ends (in 3 letter abbreviation) :type end_month: str
        :param end_year: The year where the time span ends (YYYY) :type end_year: int
        :param save_file: A boolean that declares whether the output file should be saved or not. :type save_file: bool

        Args:
            start_month (str): The month where the time span begins (in 3 letter abbreviation)
            start_year (int): The year where the time span begins (YYYY)
            end_month (str): The month where the time span ends (in 3 letter abbreviation)
            end_year (int): The year where the time span ends (YYYY)
            save_file (bool): A boolean that declares whether the output file should be saved or not.
        """
        self.start_month = start_month
        self.start_year = start_year
        self.end_month = end_month
        self.end_year = end_year
        self.file_type_preference = OUTPUT_TYPE
        self.output_file = OUTPUT_NAME
        self.output_file_path = OUTPUT_FOLDER
        self.summation_file = SUMMATION_FILE
        self.invalid_titles = ['No Post Available', '']
        self.month_dict = {"Jan": 0, "Feb": 1, "Mar": 2, "Apr": 3, "May": 4,
                           "Jun": 5, "Jul": 6, "Aug": 7, "Sep": 8, "Oct": 9, "Nov": 10, "Dec": 11}
        self.should_save_file = save_file
        self.difference = self.difference_between_dates(
            start_month, start_year, end_month, end_year)
        self.data = self.get_data(self.output_file)
        self.formatted_dataframe = self.create_dataframe()

    @staticmethod
    def get_data(file_name: str):
        """This method gets all of the data from the JSON file and returns it as
        a list.

        Args:
            file_name (str): The path to the file
        """
        with open(file_name, 'r') as f:
            data = json.load(f)
            print(len(data))
        return data

    @staticmethod
    def get_key(val, my_dict):
        """This method returns the key from the dictionary based on the value
        passed into the method.

        Args:
            val (int): The value of the key
            my_dict (dict): The dictionary to be searched

        Returns:
            (str): Returns the string "Key doesn't exist" if the key doesn't
            exist.
        """
        for key, value in my_dict.items():
            if val == value:
                return key
        return "key doesn't exist"

    @staticmethod
    def get_title_table(post: dict) -> tuple:
        """This method returns a tuple object containing the title and the table
        from the post dictionary.

        Args:
            post (dict): The dictionary from the Output File

        Returns:
            tuple: Contains the title and the table of the post.
        """
        title = post['Title']
        table = post['Table']
        return title, table

    def difference_between_dates(self, start_month: str, start_year: int, end_month: str, end_year: int) -> int:
        """Calculate the number of months between two dates.

        Args:
            start_month (str): The start month for Summation
            start_year (int): The start year for Summation
            end_month (str): The end month for Summation
            end_year (int): The end year for Summation

        Returns:
            int: An int representing the number of months between the two dates.
        """
        return (end_year - start_year) * 12 + (self.month_dict[end_month] - self.month_dict[start_month]) + 1

    def create_dataframe(self):
        """This method dynamically creates a dataframe based on the difference
        provided :returns: The dataframe that will be used later on. :rtype:
        pd.DataFrame
        """
        if self.difference < 12:
            cols = ["Article Name", "Total"]
        else:
            cols: list = [y for y in range(self.start_year, self.end_year + 1)]
            cols.insert(0, "Article Name")
        data = {i: [] for i in cols}
        return pd.DataFrame(data)

    def get_months_total(self, row: list, table: pd.DataFrame) -> list:
        """This method calculates the total for the given time span and returns
        it as a list.

        Args:
            row (list): A list containing the title of the post
            table (pd.DataFrame): A dataframe containing the data for the given
                time span

        Returns:
            row (list): A list containing the total views for the given time
            span
        """
        table = pd.DataFrame(table)
        table_col = table.iloc[self.start_year - 2012].T
        total = sum(
            table_col.iloc[month_idx]
            for month_idx in range(
                self.month_dict[self.start_month] + 1,
                self.month_dict[self.end_month] + 2,
            )
        )

        row.append(total)
        return row

    def get_one_year_total(self, row: list, table: pd.DataFrame) -> list:
        """This method calculates the total views for one year scenarios

        Args:
            row (list): The row containing the title of the post.
            table (pd.DataFrame): The table containing the data from the Output
                File

        Returns:
            list: Contains the title name and the total views for the given time
            span.
        """
        table = pd.DataFrame(table)
        start_col = table.iloc[self.start_year - 2012].T
        if self.start_year == self.end_year:
            end_col = table.iloc[self.start_year - 2012].T
        else:
            end_col = table.iloc[self.end_year - 2012].T
        total = sum(
            start_col.iloc[month_idx]
            for month_idx in range(self.month_dict[self.start_month] + 1, 13)
        )

        if self.start_year != self.end_year:
            for month_idx in range(self.month_dict[self.end_month] + 1, 1, -1):
                total += end_col.iloc[month_idx]
        row.append(total)
        return row

    def get_multi_year_total(self, row: list, table: pd.DataFrame) -> list:
        """This method calculates the total views for multiple year scenarios

        Args:
            row (list): The row containing the title of the post.
            table (pd.DataFrame): The table containing the data from the Output
                File

        Returns:
            list: Contains the title name and the total views for the given time
            span.
        """
        total = 0
        table = pd.DataFrame(table)
        for year in range(self.start_year, self.end_year + 1):
            start_col = table.iloc[year - 2012].T
            for month_idx in range(self.month_dict[self.start_month] + 1, 13):
                total += start_col.iloc[month_idx]
            row.append(total)
        return row

    def get_irregular_time_total(self, row: list, table: pd.DataFrame) -> list:
        """This method calculates the total views for multiple year scenarios

        Args:
            row (list): The row containing the title of the post.
            table (pd.DataFrame): The table containing the data from the Output
                File

        Returns:
            list: Contains the title name and the total views for the given time
            span.
        """
        total = 0
        table = pd.DataFrame(table)
        for year in range(self.start_year, self.end_year):
            start_col = table.iloc[year - 2012].T
            if year == self.start_year:
                for month_idx in range(self.month_dict[self.start_month] + 1, 13):
                    total += start_col.iloc[month_idx]
            else:
                for month_idx in range(1, 13):
                    total += start_col.iloc[month_idx]
            row.append(total)
        end_col = table.iloc[self.end_year - 2012].T
        for month_idx in range(self.month_dict[self.end_month] + 1, 1, -1):
            total += end_col.iloc[month_idx]
        row.append(total)
        return row

    def format_data(self) -> pd.DataFrame():
        """This method formats the data into the required format for
        summation.
        """
        for post in self.data:
            title, table = self.get_title_table(post)
            if title not in self.invalid_titles:
                row_data = [title]
                if self.difference < 12:
                    row_data = self.get_months_total(row_data, table)
                elif self.difference % 12 == 0 and self.start_year == self.end_year:
                    row_data = self.get_one_year_total(row_data, table)
                elif self.difference % 12 == 0:
                    row_data = self.get_multi_year_total(row_data, table)
                elif self.difference > 12:
                    row_data = self.get_irregular_time_total(row_data, table)
                self.formatted_dataframe.loc[len(
                    self.formatted_dataframe)] = row_data
        if self.difference < 12:
            self.formatted_dataframe.sort_values(
                by=['Total'], ascending=False, inplace=True)
        else:
            self.formatted_dataframe.sort_values(
                by=[self.start_year], ascending=False, inplace=True)

    def decide_subtraction(self):
        """This method decides when to subtract the data for each of the year columns
        """
        if self.difference >= 12:
            self.subtract_column_values()

    def subtract_column_values(self):
        """This method subtracts the values from the previous year from the
        current year.
        """
        cols = list(self.formatted_dataframe.columns[1:])
        self.formatted_dataframe["{}".format(
            cols[0])] = self.formatted_dataframe[cols[0]]
        for idx in range(len(cols) - 1):
            self.formatted_dataframe["{}".format(cols[idx + 1])] = self.formatted_dataframe[cols[idx + 1]] - \
                self.formatted_dataframe[
                cols[idx]]
        self.formatted_dataframe.drop(self.formatted_dataframe.columns[1:int(
            ((-len(self.formatted_dataframe.columns))+1)/2)], axis=1, inplace=True)
        self.formatted_dataframe.reset_index(drop=True, inplace=True)

    def save_file(self) -> None:
        """This method saves the data into a file based on the file type."""
        if self.file_type_preference == "csv":
            self.summation_file += '.{}'.format(
                self.file_type_preference.lower())
            self.formatted_dataframe.to_csv(self.summation_file)
        elif self.file_type_preference == "json":
            self.summation_file += '.{}'.format(
                self.file_type_preference.lower())
            with open(self.summation_file, 'w') as f:
                json.dump(self.formatted_dataframe.to_dict(
                    orient='records'), f, indent=4)
        elif self.file_type_preference == "excel":
            self.summation_file += '.xlsx'
            self.formatted_dataframe.to_excel(self.summation_file)
        else:
            raise ValueError("Output type not supported or incorrect format.")
        print("Saved at {}".format(self.summation_file))

    def main(self):
        """This method is the main method that runs the program."""
        self.format_data()
        self.decide_subtraction()
        if self.should_save_file:
            self.save_file()


if __name__ == '__main__':
    summation = Summation('Jan', 2012, 'Dec', 2020, True)
    summation.main()



# noinspection PyUnresolvedReferences
import json
from datetime import datetime
from time import sleep
from tqdm import tqdm
from colorama import Fore

import pandas as pd
from constants import *
from selenium import common, webdriver
from webdriver_manager.chrome import ChromeDriverManager


class Worker:
    def __init__(self, namespace, position: int, c_username: str, c_password: str, list_data: list):
        self.namespace = namespace
        self.save_worker_files = SAVE_WORKER_FILES
        self.progress_bar_position = position
        self.chrome_options = None
        self.driver = None
        self.data_list = list_data
        self.current_id = self.update_id()
        self.get_driver()
        self.sign_in_url = self.compile_traffic_url()
        self.log_in(c_username, c_password)
        self.button_submit = None
        self.field_email = None
        self.field_password = None
        self.run_program = True
        self.download_url = None
        self.table_df = pd.DataFrame(
            {'Year': [], 'Jan': [], 'Feb': [], 'Mar': [], 'Apr': [], 'May': [], 'Jun': [], 'Jul': [], 'Aug': [],
             'Sep': [], 'Oct': [], 'Nov': [], 'Dec': [], 'Total': []})

        self.invalid_posts = ["No Post Available", "", "Auto Draft"]
        self.post_title = None
        self.table = None
        self.empty_df = pd.DataFrame()
        self.process_pid = os.getpid()
        self.output_file_path = self.setup_output_file(OUTPUT_FOLDER)
        self.number_of_total_rows = None
        self.invalid_ids = []
        self.invalid_ids_file_path = INVALID_IDS_FILE_PATH

    def get_driver(self):
        self.chrome_options = webdriver.ChromeOptions()
        self.chrome_options.add_argument('--disable-application-cache')
        self.chrome_options.headless = True
        self.driver = webdriver.Chrome(
            ChromeDriverManager().install(), options=self.chrome_options)
        return self.driver

    def compile_traffic_url(self) -> str:
        return 'https://pharmaceuticalintelligence.wordpress.com/wp-admin/index.php?page=stats&view=' \
               'post&post={}'.format(self.current_id)

    def sign_in_email(self, username):
        self.field_email = self._extracted_from_sign_in_password_2(
            '#usernameOrEmail', username
        )

    def sign_in_password(self, password):
        self.field_password = self._extracted_from_sign_in_password_2(
            '#password', password
        )

    def _extracted_from_sign_in_password_2(self, arg0, arg1):
        result = self.driver.find_element_by_css_selector(arg0)
        result.send_keys(arg1)
        self.button_submit = self.driver.find_element_by_class_name('button')
        self.button_submit.click()
        return result

    def log_in(self, p_username: str, p_password: str):
        print('Url using to sign in', self.sign_in_url)
        self.driver.get(self.sign_in_url)
        sleep(5)
        self.sign_in_email(p_username)
        sleep(5)
        self.sign_in_password(p_password)
        sleep(3)

    @staticmethod
    def get_first_id():
        return 0

    def update_id(self):
        for element in self.data_list:
            if element['Title'] == '':
                return int(element['ID'])
        else:
            self.run_program = False

    def no_view_table(self, header, data):
        for row in data:
            row = list(map(int, row))
        num_zeros_to_add = len(header) - len(data[0])
        for row in data:
            for _ in range(num_zeros_to_add):
                row.append(0)
        self.table_df = pd.DataFrame(data, columns=header)
        self.table_df = self.table_df[self.table_df['Year'].astype(
            int) >= 2012]
        self.table_df.reset_index(drop=True, inplace=True)
        # self.table_df.set_index('Year', inplace=True)

    def format_table(self, table):
        lines = table.split('\n')
        header = ("Year " + lines[0]).split(" ")
        data = lines[1:]
        data = [row.split(' ') for row in data]
        if int(data[0][0]) < 2012:
            self.no_view_table(header, data)
        else:
            self._extracted_from_format_table_9(data, header)
        self.table_df.replace(',', '', regex=True, inplace=True)
        self.table_df = self.table_df.astype('int')
        self.table_df.sort_values(by=['Year'], inplace=True)
        self.table_df.reset_index(drop=True, inplace=True)

    def _extracted_from_format_table_9(self, data, header):
        tmp = []
        for idx in range(len(data)):
            length_of_row = len(data[idx])
            if length_of_row < len(header) and len(data) == 1:
                current_month = datetime.now().month
                num_of_0_to_add_before = (current_month + 2) - length_of_row
                tmp_2 = data[idx]
                for idx_2 in range(1, num_of_0_to_add_before + 1):
                    tmp_2.insert(idx_2, 0)
                num_of_0_to_add_after = len(header) - len(data[idx])
                for _ in range(len(data[idx]) - num_of_0_to_add_after, len(data[idx])):
                    tmp_2.insert(len(data[idx]) - 1, 0)
                tmp.append(tmp_2)
            elif length_of_row < len(header) and idx == 0:
                num_of_0_to_add = len(header) - length_of_row
                tmp_2 = data[idx]
                for idx_2 in range(1, num_of_0_to_add + 1):
                    tmp_2.insert(idx_2, 0)
                tmp.append(tmp_2)
            elif length_of_row < len(header) and idx == len(data) - 1:
                tmp_2 = data[idx]
                num_of_0_to_add = len(header) - length_of_row
                for _ in range(length_of_row - num_of_0_to_add, length_of_row):
                    tmp_2.insert(length_of_row - 1, 0)
                tmp.append(tmp_2)
            else:
                tmp.append(data[idx])
        data = tmp
        self.number_of_total_rows = int(data[0][0]) - 2012
        for years_to_add in range(1, self.number_of_total_rows + 1):
            remaining_year = int(data[0][0]) - years_to_add
            filler_list = [remaining_year]
            for _ in range(len(data[0]) - 1):
                filler_list.append(0)
            data.append(filler_list)
        for idx in range(len(data)):
            self.table_df.loc[idx] = data[idx]

    def check_post_title(self):
        title_list = self.post_title.split('.')
        if len(title_list) == 1:
            return True
        elif len(title_list[-1]) == 0:
            return True
        else:
            return False

    def append_data(self, p_id: int, title: str, table):
        if isinstance(table, pd.DataFrame):
            table = table.to_dict()
        for element in self.data_list:
            if int(element['ID']) == p_id:
                element['Title'] = title
                element['Table'] = table

    def scrape_title_and_table(self):
        try:
            self.download_url = self.compile_traffic_url()
            self.driver.get(self.download_url)
            sleep(2)
            self.post_title = self.driver.find_element_by_class_name(
                'statscolumn').text.split("\n")[0][11:]
            self.table = self.driver.find_elements_by_xpath("//table")
            if len(self.table) == 4:
                self.table = self.table[1].text
            else:
                self.table = self.table[0].text
            self.format_table(self.table)
            if self.post_title != '':
                self.append_data(
                    self.current_id, self.post_title, self.table_df)
                self.current_id = self.update_id()
            else:
                self.invalid_ids.append(self.current_id)
                self.append_data(
                    self.current_id, "No Post Available", dict(self.empty_df))
                self.current_id = self.update_id()
        except (common.exceptions.NoSuchElementException, common.exceptions.WebDriverException):
            pass
        except IndexError:
            self.invalid_ids.append(self.current_id)
            self.append_data(
                self.current_id, "No Post Available", dict(self.empty_df))
            self.current_id = self.update_id()

    def main(self):
        with tqdm(total=len(self.data_list), bar_format="{l_bar}%s{bar}%s{r_bar}" % (Fore.BLUE, Fore.RESET), position=self.progress_bar_position) as pbar:
            pbar.set_description(
                "[Worker ID: {}]Extracting Data: ".format(self.process_pid))
            while self.run_program:
                self.scrape_title_and_table()
                pbar.update(1)
        pbar.close()
        self.driver.close()
        self.save_data_list(self.data_list)
        if len(self.invalid_ids) > 0:
            self.save_invalid_ids()

    def save_invalid_ids(self):
        with open(self.invalid_ids_file_path + '/invalid_ids_{}'.format(self.process_pid) + ".txt", 'w') as f:
            for idx in self.invalid_ids:
                f.write(str(idx) + '\n')

    def setup_output_file(self, output_file_path: str) -> str:
        return output_file_path + '/output_{}'.format(self.process_pid) + ".json"

    def save_data_list(self, data):
        """Save the data into a json file or in the namespace depending on the boolean"""
        if self.save_worker_files:
            with open(self.output_file_path, 'w+') as outfile:
                outfile.seek(0)
                json.dump(data, outfile)
        else:
            manager_namespace = self.namespace.data
            manager_namespace.extend(data)
            self.namespace.data = manager_namespace


if __name__ == '__main__':
    import os
    import sys
    sys.path.append('../../')
    data_list = [
        {
            "ID": "20552",
            "Title": "",
            "Table": ""
        },
        {
            "ID": "20566",
            "Title": "",
            "Table": ""
        },
        {
            "ID": "20560",
            "Title": "",
            "Table": ""
        }
    ]
    # noinspection PyArgumentList
    worker_inst = Worker(None, 0, "", "", data_list)
    worker_inst.main()



import termios
import warnings

import getpass4

from dateutil import parser
from Auto_Data_Removal.auto_data_removal import AutoDataRemoval
from Email_Sender.email_sender import EmailSender
from Summation.summation_main import SummationMain
from Webscraping.webscrape_data import WebscrapeData
from Webscraping.webscrape_ids import WebscrapeIDs

class LpbiViewsExtraction:

    def __init__(self):
        pass

    @staticmethod
    def get_date():
        """Get the date from the user.

        Returns:
            A datetime.date object.
        """
        date_input = input("Enter a date in the format MM/DD/YYYY or YYYY: ")
        date = list(map(int, date_input.split('/')))
        if len(date) == 1 and date[0] == 2012:
            return parser.parse("04/01/{}".format(date[0]))
        else:
            return parser.parse(date_input)

    @staticmethod
    def get_user_credentials() -> tuple:
        f_username = input("Enter your Username: ")
        f_password = ''
        try:
            f_password = getpass4.getpass(prompt="Enter your Password: ")
        except termios.error:
            print("", end='\r')
            print("Note: Your password is not hidden.")
            f_password = input("Enter your Password: ")
        finally:
            return f_username, f_password

    def main(self):
        warnings.filterwarnings('ignore')
        auto_data_removal = AutoDataRemoval()
        auto_data_removal.main()

        start_date = self.get_date()
        end_date = self.get_date()

        v_username, v_password = self.get_user_credentials()

        scrape_ids_inst = WebscrapeIDs(v_username, v_password)
        scrape_ids_inst.main()

        main_inst = WebscrapeData(v_username, v_password)
        main_inst.main()

        summation_inst = SummationMain(start_date, end_date)
        summation_inst.main()

        EmailSender()


if __name__ == '__main__':
    LpbiViewsExtraction.main()



import sys

from colorama import Fore, Style
from simple_term_menu import TerminalMenu

from Assets.lpbi_views_extraction import LpbiViewsExtraction
from Assets.setup_constants import SetupConstants

class ViewsExtraction:
    
    def __init__(self):
        print(Fore.BLUE)
        self.menu_options = ["Setup", "Extract Views", "Exit"]
        self.user_selection = self.get_user_selection()

    def get_user_selection(self):
        terminal_menu = TerminalMenu(self.menu_options, title="Hello there, \nPlease choose an action from the options below: ")
        return terminal_menu.show()

    def run_setup_constants(self):
        setup_constants_inst = SetupConstants()
        setup_constants_inst.main()

    @staticmethod
    def run_lpbi_views_extraction():
        lpbi_inst = LpbiViewsExtraction()
        lpbi_inst.main()
    
    def main(self):
        try:
            if self.user_selection == 0:
                self.run_setup_constants()
            elif self.user_selection == 1:
                self.run_lpbi_views_extraction()
            elif self.user_selection == 2:
                sys.exit(0)
        except (KeyboardInterrupt, EOFError, SystemExit, TypeError):
            sys.exit(0)

if __name__ == '__main__':
    ViewsExtraction().main()
    print(Style.RESET_ALL)



import json
import math
import multiprocessing
import sys
from time import time

from Assets.Webscraping.worker import Worker
from constants import *


class WebscrapeData:
    def __init__(self, username, password):
        self.save_worker_files = SAVE_WORKER_FILES
        self.manager, self.namespace = self.create_manager()
        self.namespace.data = []
        self.number_of_workers = self.generate_number_of_workers()
        self.json_file_path = OUTPUT_NAME
        self.json_file = self.read_json()
        self.all_chunks, self.number_of_dictionaries = self.split_json()
        self.username, self.password = (username, password)
        self.worker_instances = []
        self.workers = self.create_number_of_workers()
        self.all_lists = None
        self.process_file_path = OUTPUT_FOLDER

    @staticmethod
    def create_manager():
        manager = multiprocessing.Manager()
        namespace = manager.Namespace()
        return manager, namespace

    @staticmethod
    def generate_number_of_workers() -> int:
        return NUMBER_OF_THREADS

    def read_json(self):
        with open(self.json_file_path, 'r') as file:
            data = json.load(file)
        return data

    def split_json(self) -> list:
        number_of_dictionaries = math.ceil(
            len(self.json_file) / self.number_of_workers)
        dictionaries = self.json_file
        list_of_chunks = []
        [list_of_chunks.append(dictionaries[chunk_idx:chunk_idx + number_of_dictionaries])
         for chunk_idx in range(0, len(self.json_file), number_of_dictionaries)]
        return list_of_chunks, len(self.json_file)

    def create_number_of_workers(self):
        try:
            return [multiprocessing.Process(
                target=self.worker_main,
                args=(
                    self.namespace,
                    worker,
                    self.username,
                    self.password,
                    self.all_chunks[worker]
                )
            ) for worker in range(self.number_of_workers)]
        except IndexError:
            print("[IndexError] There are more workers than data.")
            print("Try reducing the number of workers.")
            sys.exit()

    @staticmethod
    def worker_main(namespace, position: int, username: str, password: str, json_list: list):
        worker_inst = Worker(namespace, position,
                             username, password, json_list)
        worker_inst.main()

    def start_workers(self):
        for worker in self.workers:
            worker.start()

    def join_workers(self):
        for worker in self.workers:
            worker.join()

    @staticmethod
    def get_process_files(folder_path):
        """Gather all of the json file paths into one list from the output folder"""
        return [
            os.path.join(folder_path, file)
            for file in os.listdir(folder_path)
            if file.startswith("output_") and file.endswith(".json")
        ]

    def retrieve_lists(self) -> list:
        """Return all the lists from Manager's namespace or from worker files."""
        if self.save_worker_files:
            all_data = []
            all_files = self.get_process_files(self.process_file_path)
            for file in all_files:
                with open(file, 'r') as file_inst:
                    data = json.load(file_inst)
                    for post in data:
                        all_data.append(post)
        else:
            all_data = self.namespace.data
        return all_data

    def save_json(self):
        with open(self.json_file_path, 'w+') as file:
            json.dump(self.all_lists, file, indent=4)

    def main(self):
        program_start_time = time()
        self.start_workers()
        self.join_workers()
        self.all_lists = self.retrieve_lists()
        self.save_json()
        program_end_time = time()
        print("Webscraping Data took {} seconds to execute.".format(
            program_end_time - program_start_time))


if __name__ == '__main__':
    start_time = time()
    main_inst = WebscrapeData("", "")
    main_inst.main()
    end_time = time()
    print("The program took {} seconds to execute.".format(end_time - start_time))



import datetime
import os
import smtplib
import ssl
from email import encoders
from email.mime.base import MIMEBase
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.utils import formatdate
from constants import *


class EmailSender:

    def __init__(self):
        self.summation_file = self.get_file()
        self.msg = None
        self.sender_email = None
        self.receiver_email = None
        self.password = None
        self.body = self.read_body()
        self.context = None
        self.establish_email_params()
        self.bool = self.send_email()

    @staticmethod
    def read_body():
        with open(os.path.dirname(os.path.realpath(__file__)) + "/Template/summation_file.txt") as file:
            return file.read()

    def get_file(self):
        initial_name = SUMMATION_FILE
        if OUTPUT_TYPE in ["csv", "json"]:
            initial_name += '.{}'.format(self.file_type_preference.lower())
        elif OUTPUT_TYPE == "excel":
            initial_name += '.xlsx'
        else:
            print("Invalid format given in Constants")
        return initial_name

    def establish_email_params(self):
        self.msg = MIMEMultipart()
        self.sender_email = EMAIL_FROM
        self.receiver_email = RECIPIENT
        self.password = PASSWORD
        self.msg['From'] = self.sender_email
        self.msg['To'] = self.receiver_email
        self.msg['Date'] = formatdate(localtime=True)
        self.msg['Subject'] = 'Here is the LPBI Views File for {}'.format(
            datetime.date.today())

    def send_email(self):
        email_sent_status = False
        part = MIMEBase('application', "octet-stream")
        part.set_payload(open(self.summation_file, "rb").read())
        encoders.encode_base64(part)
        part.add_header('Content-Disposition', 'attachment',
                        filename=self.summation_file.split('/')[-1])
        self.msg.attach(part)
        self.msg.attach(MIMEText(self.body, "plain"))
        self.context = ssl.create_default_context()
        try:
            with smtplib.SMTP_SSL("smtp.gmail.com", 465, context=self.context) as server:
                server.login(self.sender_email, self.password)
                server.sendmail(self.sender_email, self.receiver_email.split(
                    ","), self.msg.as_string())
        except Exception as e:
            print(type(e).__name__ + ': ' + str(e))
        else:
            email_sent_status = True
        finally:
            return email_sent_status


if __name__ == "__main__":
    EmailSender()



import os
import psutil

OUTPUT_NAME: str = os.path.join(os.path.dirname(
    os.path.realpath(__file__))) + "/Output/data.json"
OUTPUT_FOLDER: str = os.path.join(
    os.path.dirname(os.path.realpath(__file__))) + '/Output'
SUMMATION_FILE: str = os.path.join(os.path.dirname(
    os.path.realpath(__file__))) + '/Output/Summation'
INVALID_IDS_FILE_PATH: str = os.path.join(os.path.dirname(
    os.path.realpath(__file__))) + '/Output/Invalid_Ids'
SAVE_WORKER_FILES: bool = True
NUMBER_OF_THREADS: int = psutil.cpu_count() * 2
ALTERABLE_CONSTANTS = True
OUTPUT_TYPE: str = 'excel'  # Options: csv, excel, json
RECIPIENT: str = 'abhisar.muz@gmail.com, srinivassriram06@gmail.com'
EMAIL_FROM: str = 'ereaders.visitors@gmail.com'
PASSWORD: str = ''



import json
from time import sleep, time

from constants import *
from selenium import common, webdriver
from tqdm import trange
from colorama import Fore
# noinspection PyUnresolvedReferences
from webdriver_manager.chrome import ChromeDriverManager


class WebscrapeIDs:
    def __init__(self, v_username, v_password):
        self.extract_views_url = 'https://pharmaceuticalintelligence.com/wp-admin/edit.php?post_status=publish&post_type=post'
        self.chrome_options = None
        self.driver = None
        self.download_url = None
        self.field_email = None
        self.field_password = None
        self.button_submit = None
        self.button = None
        self.c_username = v_username
        self.c_password = v_password
        self.run_program = True
        self.output_file = OUTPUT_NAME
        self.post_limit = 0

    def get_driver(self):
        self.chrome_options = webdriver.ChromeOptions()
        self.chrome_options.add_argument('--disable-application-cache')
        self.chrome_options.headless = True
        self.driver = webdriver.Chrome(
            ChromeDriverManager().install(), options=self.chrome_options)
        return self.driver

    def signing_in(self, driver, p_username: str, p_password: str):
        print('Url using to sign in', self.extract_views_url)
        driver.get(self.extract_views_url)
        sleep(5)
        self.field_email = driver.find_element_by_css_selector(
            '#usernameOrEmail')
        self.field_email.send_keys(p_username)
        self.button_submit = driver.find_element_by_class_name('button')
        self.button_submit.click()
        sleep(5)
        self.field_password = driver.find_element_by_css_selector('#password')
        self.field_password.send_keys(p_password)
        self.button_submit = driver.find_element_by_class_name('button')
        self.button_submit.click()
        sleep(3)

    @staticmethod
    def get_total_num_of_articles(driver):
        return int(driver.find_element_by_class_name('displaying-num').text.split(" ")[0].replace(",", ""))

    @staticmethod
    def set_parameters(driver):
        screen_options_button = driver.find_element_by_class_name(
            'show-settings')
        screen_options_button.click()
        sleep(1)
        num_of_items = driver.find_element_by_class_name('screen-per-page')
        num_of_items.clear()
        num_of_items.send_keys(999)  # LLJW and Srini
        apply_button = driver.find_element_by_name('screen-options-apply')
        apply_button.click()

    def save_output(self, p_id: int):
        content = {"ID": p_id, "Title": '', "Table": ''}
        with open(self.output_file, 'r+') as file:
            file_data = json.load(file)
            file_data.append(content)
            file.seek(0)
            json.dump(file_data, file, indent=4)

    def scrape(self, driver):
        driver.get(self.extract_views_url)
        print("Sleeping 2 Seconds...")
        sleep(2)
        self.post_limit = self.get_total_num_of_articles(driver)
        self.set_parameters(driver)
        sleep(5)
        with trange(self.post_limit, bar_format="{l_bar}%s{bar}%s{r_bar}" % (Fore.BLUE, Fore.RESET)) as pbar:
            pbar.set_description("Extracting IDs: ")
            try:
                current_num_of_posts = 0
                while current_num_of_posts <= self.post_limit:
                    elements = driver.find_elements_by_xpath(
                        "//td[@class = 'stats column-stats']//a")
                    for element in elements:
                        link = element.get_attribute('href')
                        post_id = link.split("=")[-1]
                        self.save_output(post_id)
                        pbar.update(1)
                        current_num_of_posts += 1
                    next_button = driver.find_elements_by_xpath(
                        "//a[@class = 'next-page button']")
                    next_page_link = ''
                    for element in next_button:
                        next_page_link = element.get_attribute('href')
                    try:
                        driver.get(next_page_link)
                    except common.exceptions.InvalidArgumentException:
                        pass
                if pbar.n >= self.post_limit:
                    pbar.set_description("Extraction Complete: ")
                    pbar.bar_format = "{l_bar}%s{bar}%s{r_bar}" % (
                        Fore.GREEN, Fore.RESET)
            except (common.exceptions.NoSuchElementException, IndexError) as e:
                print("Error: \n{}".format(e))
                pbar.set_description("Error Occurred: ")
                pbar.bar_format = "{l_bar}%s{bar}%s{r_bar}" % (
                    Fore.RED, Fore.RESET)

    def main(self):
        program_start_time = time()
        self.driver = self.get_driver()
        self.signing_in(self.driver, p_username=self.c_username,
                        p_password=self.c_password)
        self.scrape(self.driver)
        self.driver.quit()
        program_end_time = time()
        print("Webscraping Ids took {} seconds to execute.".format(
            program_end_time - program_start_time))


if __name__ == '__main__':
    start_time = time()
    scrape_ids_inst = WebscrapeIDs("", "")
    scrape_ids_inst.main()
    end_time = time()
    print("The program took {} seconds to execute.".format(end_time - start_time))



from time import time

from Assets.Summation.summation import Summation


class SummationMain:
    def __init__(self, start_date, end_date):
        self.start_date = start_date
        self.end_date = end_date
        self.difference = self.difference_between_dates(
            self.start_date, self.end_date)
        self.month_dict = {"Jan": 1, "Feb": 2, "Mar": 3, "Apr": 4, "May": 5,
                           "Jun": 6, "Jul": 7, "Aug": 8, "Sep": 9, "Oct": 10, "Nov": 11, "Dec": 12}

    @staticmethod
    def get_key(val, my_dict):
        """This method returns the key from the dictionary based on the value passed into the method.

        Args:
            val (int): The value of the key
            my_dict (dict): The dictionary to be searched

        Returns:
            (str): Returns the string "Key doesn't exist" if the key doesn't exist.
        """
        for key, value in my_dict.items():
            if val == value:
                return key
        return "key doesn't exist"

    @staticmethod
    def difference_between_dates(start_date, end_date) -> int:
        """Calculate the number of months between two dates.

        Args:
            start_date: A datetime.date object.
            end_date: A datetime.date object.

        Returns:
            An int representing the number of months between the two dates.
        """
        return (end_date.year - start_date.year) * 12 + (end_date.month - start_date.month)

    def main(self):
        """Main method.
        """
        program_start_time = time()
        summation_inst = Summation(self.get_key(self.start_date.month, self.month_dict), self.start_date.year,
                                   self.get_key(self.end_date.month, self.month_dict), self.end_date.year, True)
        summation_inst.main()
        program_end_time = time()
        print("Summation program took {} seconds to execute.".format(
            program_end_time - program_start_time))


if __name__ == '__main__':
    from dateutil import parser

    start_time = time()
    summation_main_inst = SummationMain(parser.parse(
        "04/01/2012"), parser.parse("06/30/2020"))
    summation_main_inst.main()
    end_time = time()
    print("Summation program took {} seconds to execute.".format(
        end_time - start_time))



import sys
import os
import re
from simple_term_menu import TerminalMenu

sys.path.append(os.path.dirname(os.path.realpath(__file__)))


class SetupConstants:
    def __init__(self):
        self.constants_path = os.path.dirname(
            os.path.realpath(__file__)) + "/../constants.py"
        self.orig_dict_options = None
        self.menu_options = self.format_constant_file()
        self.user_selection = self.get_user_selection()

    def get_user_selection(self):
        terminal_menu = TerminalMenu(
            self.menu_options, title="What would you like to change: ")
        return terminal_menu.show()

    def read_constants_file(self):
        with open(self.constants_path, "r") as constants_file:
            constants_file_contents = constants_file.readlines()
        return constants_file_contents

    def parse_lines(self, line):
        line = line.replace('\n', '')
        line, _, _ = line.partition('#')
        if 'import ' in line or line == '':
            return None
        return line

    def get_only_alterable_constants(self, line_list):
        return line_list[line_list.index('ALTERABLE_CONSTANTS = True') + 1:]

    def make_dict_for_constants(self, line_list):
        constants_dict = {}
        for lines in line_list:
            split_line = list(
                filter(None, re.sub(':.*?=', '', lines).split(' ')))
            if len(split_line) == 2:
                constants_dict[split_line[0]] = split_line[1]
            elif len(split_line) > 2:
                constants_dict[split_line[0]] = ' '.join(split_line[1:])
            else:
                raise Exception('Invalid line in constants file: ' + lines)
        return constants_dict

    def format_constant_file(self):
        constants_file_contents = self.read_constants_file()
        formatted_lines = []
        for line in constants_file_contents:
            result = self.parse_lines(line)
            if result is not None:
                formatted_lines.append(result)
        formatted_lines = self.get_only_alterable_constants(formatted_lines)
        self.orig_dict_options = self.make_dict_for_constants(formatted_lines)
        formatted_options = [option.capitalize()
                             for option in self.orig_dict_options.keys()]
        formatted_options.append("Exit")
        return formatted_options

    def set_output_type(self, str_output_type):
        self.orig_dict_options['OUTPUT_TYPE'] = str_output_type.lower()

    def output_type_menu(self):
        menu_options = ['CSV', 'Excel', 'JSON']
        terminal_menu = TerminalMenu(
            menu_options, title="Choose an output file type: ")
        self.set_output_type(menu_options[terminal_menu.show()])

    def set_recipient_email(self, email_str):
        self.orig_dict_options['RECIPIENT'] = email_str[:-2]

    def recipient_menu(self):
        number_of_recipients = int(
            input("How many recipients do you wish to send the output file to: "))
        recipient_email_str = ''
        for _ in range(number_of_recipients):
            recipient_email = input("Enter recipient email: ")
            recipient_email_str += recipient_email + ', '
        self.set_recipient_email(recipient_email_str)

    def set_email_from(self):
        self.orig_dict_options['EMAIL_FROM'] = input(
            "Enter the email you would like to use for sending the output file: ")

    def set_password(self):
        self.orig_dict_options['PASSWORD'] = input(
            "Enter the password for the email you wish to use: ")

    def determine_action(self):
        if self.menu_options[self.user_selection] == 'Output_type':
            self.output_type_menu()
        elif self.menu_options[self.user_selection] == 'Recipient':
            self.recipient_menu()
        elif self.menu_options[self.user_selection] == 'Email_from':
            self.set_email_from()
        elif self.menu_options[self.user_selection] == 'Password':
            self.set_password()
        elif self.menu_options[self.user_selection] == 'Exit':
            sys.exit(0)

    def write_constants_file(self):
        constants_file_content = self.read_constants_file()
        with open(self.constants_path, "w") as constants_file:
            for key, value in self.orig_dict_options.items():
                for line_idx in range(len(constants_file_content)):
                    line = constants_file_content[line_idx]
                    if key in line:
                        line = line.split('\'')
                        line = line[0] + '\'' + \
                            value.replace('\'', '') + '\'' + line[2]
                        constants_file_content[line_idx] = line
            constants_file.writelines(constants_file_content)

    def main(self):
        try:
            self.determine_action()
            self.write_constants_file()
        except (KeyboardInterrupt, EOFError, SystemExit, TypeError):
            sys.exit(0)


if __name__ == '__main__':
    setup_constant_inst = SetupConstants()
    setup_constant_inst.main()



from os import listdir
from os.path import isfile, join
from simple_term_menu import TerminalMenu

from constants import *


class AutoDataRemoval:
    def __init__(self):
        self.path_to_dir = OUTPUT_FOLDER
        self.files = [f for f in listdir(
            self.path_to_dir) if isfile(join(self.path_to_dir, f))]
        self.num_of_files = len(self.files)
        self.question = None

    @staticmethod
    def create_output_file():
        print("No Files to Delete\nCreating Output File")
        with open(OUTPUT_NAME, "w") as output_file:
            output_file.write("[]")
        print("Output File Created")

    def get_user_confirmation(self):
        menu_options = ["[Y] Yes", "[N] No"]
        terminal_menu = TerminalMenu(
            menu_options, title="There are {} files in the Output Folder. Do you want to delete them: ".format(self.num_of_files))
        menu_entry_index = terminal_menu.show()
        self.question = menu_options[menu_entry_index][1].lower()

    def delete_files(self, response):
        if response == 'y':
            for file_path in self.files:
                os.remove(self.path_to_dir + "/" + file_path)
            self.create_output_file()
        else:
            print("No Files Deleted")

    def main(self):
        if self.num_of_files > 0:
            self.get_user_confirmation()
            self.delete_files(self.question)
        elif self.num_of_files == 0:
            self.create_output_file()


if __name__ == '__main__':
    auto_data_removal = AutoDataRemoval()
    auto_data_removal.main()



import os.path
from setuptools import setup

HERE = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(HERE, "README.md")) as fid:
    README = fid.read()

setup(
    name="views_extraction",
    version="1.1.0",
    description="This project extracts the view data of every post from WordPress.",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/AbhisarAnand/LPBI-Data-Extraction-Project",
    author="Abhisar Anand, Srinivas Sriram",
    author_email="abhisar.muz@gmail.com, srinivassriram06@gmail.com",
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
    ],
    packages=["views_extraction"],
    include_package_data=True,
    install_requires=[
        "Cython", "getpass4", "selenium", "pandas", "webdriver-manager", "numpy", "python-dateutil", "setuptools", "openpyxl", "tqdm", "psutil", "colorama"
    ],
    entry_points={"console_scripts": [
        "views_extraction=views_extraction.__main__:main"]},
)
